package cloud

import (
	"context"
	"fmt"
	"strings"

	corev1 "k8s.io/api/core/v1"
	"quark.netapp.io/volume-controller/pkg/quarkconfig"
)

type CloudClient interface {
	CreateLoadBalancerAddress(ctx context.Context, name string, subnet string) error
	DeleteLoadBalancerAddress(ctx context.Context, name string) error
	GetLoadBalancerAddress(ctx context.Context, name string) (string, error)
	IsNotFound(err error) bool

	ResetInstance(ctx context.Context, name string) error
	ResetInstanceForZone(ctx context.Context, name, zone string) error
	ResetInstanceForProviderID(ctx context.Context, id string) error
	AddCloudAnnotations(service *corev1.Service, ilbSubnet string, ilbGlobalAccess bool)

	ListZonalNodePools(ctx context.Context) ([]*NodePool, error)
	ListRegionalNodePools(ctx context.Context) ([]*NodePool, error)
	ResizeRegionalNodePool(ctx context.Context, nodePool string, newSize int64) error
}

type NodePool struct {
	Name        string
	MachineType string
}

type ClusterInfo struct {
	Name     string
	Location string
}

func ClientFactory(cloudProvider, credentialsPath, project, zone string, sharedVip bool) (CloudClient, error) {
	switch cloudProvider {
	case quarkconfig.CloudGCP:
		return NewGCPClientLocal(credentialsPath, project, zone, sharedVip)
	case quarkconfig.CloudAzure:
		return NewAzureClient()
	default:
		return nil, fmt.Errorf("unsupported cloud provider: %s", cloudProvider)
	}
}

func getSubnetNameFromLink(subnetLink string) string {
	split := strings.Split(subnetLink, "/")
	return split[len(split)-1]
}
